#ifndef APP_MEM_H
#define APP_MEM_H
/*
 * appmem.h - intergace to memory allocation
 * 
 * include LICENSE
 */

#ifdef HAVE_CONFIG_H
   #include <config.h>
#endif
/*
 * CENTOS : msglog include stdlib.h that need to be included before glib.h
 */
#include <msglog.h>

#ifdef HAVE_GLIB
  #include <glib.h>
  #undef malloc
  #undef realloc
  #undef free
  #define malloc  g_malloc
  #define realloc g_realloc
  #define free    g_free
  /* this is for CENTOS */
//  #define __need_malloc_and_calloc
#endif

/*
 * Macros for allocating new memory. Automatically performs type cast.
 */

/** \brief Macro for allocating memory.
 *  \param type  The C type of the memory to allocate.
 *  \param count Allocate enough memory hold count types.
 *
 *  This macro is just a wrapper for app_malloc() and should be used to avoid
 *  the repetitive task of casting the returned pointer.
 */


#define app_new(type, count)          \
    ((type *) app_malloc ((unsigned) sizeof (type) * (count)))

#define app_new0(type, count)         \
    ((type *) app_malloc0 ((unsigned) sizeof (type) * (count)))

#define app_renew(type, mem, count)   \
    ((type *) app_realloc (mem, (unsigned) sizeof (type) * (count)))

/*
 * Malloc and free wrappers. Perform sanity checks for you.
 */

extern void *app_malloc      (size_t size);
extern void *app_malloc0     (size_t size);
extern void *app_realloc     (void *ptr, size_t size);
extern char *app_strndup     (const char *s, int size);
extern unsigned int app_power_of_2(unsigned int size);
extern char *app_strdup      (const char *s);
extern int app_strlen(const char *s);
extern char *app_strcpy( char *s1, const char *s2);
extern char *app_strncpy( char *s1, const char *s2, int n);
extern int app_strcmp(const char *s1, const char *s2);
extern int app_strncmp(const char *s1, const char *s2, int n);
extern int app_strcasecmp(const char *s1, const char *s2);
extern int app_strncasecmp(const char *s1, const char *s2, int n);
extern void  app_free        (void *ptr);

char *app_strstr(const char *s1, const char *s2);
char *app_strcasestr(const char *s1, const char *s2);

void app_dup_str( char **varp,  char *str);

#endif /* APP_MEM_H */
